/******************************************************************************
 * Spine Runtimes License Agreement
 * Last updated January 1, 2020. Replaces all prior versions.
 *
 * Copyright (c) 2013-2020, Esoteric Software LLC
 *
 * Integration of the Spine Runtimes into software or otherwise creating
 * derivative works of the Spine Runtimes is permitted under the terms and
 * conditions of Section 2 of the Spine Editor License Agreement:
 * http://esotericsoftware.com/spine-editor-license
 *
 * Otherwise, it is permitted to integrate the Spine Runtimes into software
 * or otherwise create derivative works of the Spine Runtimes (collectively,
 * "Products"), provided that each user of the Products must obtain their own
 * Spine Editor license and redistribution of the Products in any form must
 * include this license and copyright notice.
 *
 * THE SPINE RUNTIMES ARE PROVIDED BY ESOTERIC SOFTWARE LLC "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ESOTERIC SOFTWARE LLC BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES,
 * BUSINESS INTERRUPTION, OR LOSS OF USE, DATA, OR PROFITS) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THE SPINE RUNTIMES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include <spine/AttachmentVertices.h>

USING_NS_CC;

namespace spine {

AttachmentVertices::AttachmentVertices (Texture2D* texture, int verticesCount, unsigned short* triangles, int trianglesCount) {
	_texture = texture;
	if (_texture) _texture->retain();

	_triangles = new TrianglesCommand::Triangles();
	_triangles->verts = new V3F_C4B_T2F[verticesCount];
	_triangles->vertCount = verticesCount;
	_triangles->indices = triangles;
	_triangles->indexCount = trianglesCount;
}

AttachmentVertices::~AttachmentVertices () {
	delete [] _triangles->verts;
	delete _triangles;
	if (_texture) _texture->release();
}

AttachmentVertices* AttachmentVertices::clone()
{
    auto avCopy = new AttachmentVertices(this->_texture, this->_triangles->vertCount, this->_triangles->indices, this->_triangles->indexCount);

    cocos2d::V3F_C4B_T2F* dstVertices = avCopy->_triangles->verts;
    cocos2d::V3F_C4B_T2F* srcVertices = this->_triangles->verts;

    for (int i = 0; i < this->_triangles->vertCount; ++i) {
        dstVertices[i].texCoords.u = srcVertices[i].texCoords.u;
        dstVertices[i].texCoords.v = srcVertices[i].texCoords.v;
    }

    return avCopy;
}

}
